'use strict';

/*
 *  data.js
 *  (factory)mambo2_compress_data
 */

angular.module('btdata').factory('mambo2_compress_data', function (localStorageService, bthf_add) {

    // condensed format [{l:<value>, lng:<how-often>},{...},...] to extended format [<value>,<value2>,...]
    function extend(data) {
        var tmpData = [];
        angular.forEach(data, function (val) {
            var c;
            if (c = val.lng) {
                while (c--) {
                    tmpData.push(val.l);
                }
            } else tmpData.push(val.l);
        });
        return tmpData;
    }

    function fetchData(tag) {
        switch (tag) {
            case "heart":
                var heartData = localStorageService.get('alt_heart_data') || {};
                return heartData;
                break;
            case "sleep":
                var sleepData = localStorageService.get('alt_sleep_data') || {};
                return sleepData;
                break;
        }
    }

    function persistData(tag, data) {
        switch (tag) {
            case "heart":
                localStorageService.set('alt_heart_data', data);
                break;
            case "sleep":
                localStorageService.set('alt_sleep_data', data);
                break;
        }
    }

    /*
     *  return format: {'YYYY-MM-DD':{'mm':<value != 0>, ...}, ...}
     */
    function compress(entry, condensed, tag) {
        if (condensed) entry.data = extend(entry.data);

        var d = moment(entry.utc);
        var s = angular.copy(d).startOf('day'); // here needs to be a copy or else d is set to startOf day
        var idx = d.format('YYYY-MM-DD');
        var offset_s = d.diff(s, 's') + 1;
        var offset_m = Math.round(offset_s / 60);
        var utc_offset = Math.round(entry.utc_offset / 60);
        var shouldParse = typeof entry.data[0] === 'string';

        var localData = fetchData(tag);
        var tmpData = localData[idx] || {};

        for (var i = 0, len = entry.data.length; i < len; i++) {
            // check if still the same day
            if (offset_m >= 1440) {
                localData[idx] = tmpData;
                persistData(tag, localData);
                idx = s.add(1, 'day').format('YYYY-MM-DD');
                tmpData = localData[idx] || {};
                offset_m -= 1440;
            }

            // add new value
            if (entry.data[i] != 0) {
                tmpData[offset_m] = shouldParse ? parseInt(entry.data[i], 16) : entry.data[i];

                // healthfit integration
                bthf_add.addData('heart', {
                    date: idx,
                    offset_m: offset_m,
                    utc_offset: utc_offset,
                    value: tmpData[offset_m]
                }

                // last heart reading if not 0
                );if (tag == "heart" && i == len - 1) {
                    var last_reading = {
                        readingHex: entry.data[i],
                        reading: parseInt(entry.data[i], 16),
                        dateStr: idx,
                        time: offset_m,
                        utc: parseInt(moment(idx).minutes(offset_m).format('X'))
                    };
                    localStorageService.set('heart_last_reading', last_reading);
                }
            }

            // add to offset_m
            offset_m += utc_offset;
        }
        localData[idx] = tmpData;
        persistData(tag, localData);
    }

    /*
     *  getByDate() to retrieve heart and sleep data from raw_entries
     *  -> do comparison of sleep and heart data where necessary
     *  -> return Object holding data and optional night data 
     *
     *  @params: tag -> either 'sleep' or 'heart'
     *  @params: date -> valid moment() input
     *  @params: options -> splitMode(bool)
     *                   -> sleepModeBy(number)
     */
    function getByDate(tag, date) {
        var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

        var result = [];
        var heartData = fetchData('heart');
        var d = moment(date).format('YYYY-MM-DD');

        if (tag == 'heart') {

            // heart data
            var d = moment(date).startOf('day');
            var idx = d.format('YYYY-MM-DD');
            var tmp_heartData = heartData[idx] || {};
            if (!options.splitMode) {

                // get heart data for whole day
                angular.forEach(Object.keys(tmp_heartData), function (key) {
                    var _d = angular.copy(d);
                    result.push({
                        val: tmp_heartData[key],
                        date: _d.minute(parseInt(key))
                    });
                });

                return { data: result };
            } else {

                // get heart data for day/night seperately
                var sleepData = fetchData('sleep');
                var tmp_sleep = sleepData[idx] || {};
                var result_night = angular.copy(result);
                var result_day = angular.copy(result);
                angular.forEach(Object.keys(tmp_heartData), function (key) {
                    var _d = angular.copy(d);
                    var sVal;
                    if (sVal = tmp_sleep[key]) {
                        if (sVal < options.sleepModeBy) {
                            result_night.push({
                                val: tmp_heartData[key],
                                date: _d.minute(parseInt(key))
                            });
                        } else {
                            result_day.push({
                                val: tmp_heartData[key],
                                date: _d.minute(parseInt(key))
                            });
                        }
                    }
                    result.push({
                        val: tmp_heartData[key],
                        date: _d.startOf('day').minutes(parseInt(key))
                    });
                });

                return { data: result || [], night: result_night || [], day: result_day || [] };
            }
        } else if (tag == 'sleep') {

            // sleep data
            var data = fetchData(tag);

            // second half of day before
            var d = moment(date).startOf('day').subtract(1, 'days');
            var idx = d.format('YYYY-MM-DD');
            var tmp_sleep_2 = data[idx] || {};
            var tmp_heart_2 = heartData[idx] || {};
            angular.forEach(Object.keys(tmp_sleep_2), function (key) {
                if (tmp_heart_2[key] && parseInt(key) >= 840) {
                    // for all values after 14:00Uhr
                    var _d = angular.copy(d);
                    result.push({
                        val: tmp_sleep_2[key],
                        date: _d.minute(parseInt(key))
                    });
                }
            });

            // first half of this day
            var d = moment(date).startOf('day');
            var idx = d.format('YYYY-MM-DD');
            var tmp_sleep_1 = data[idx] || {};
            var tmp_heart_1 = heartData[idx] || {};
            angular.forEach(Object.keys(tmp_sleep_1), function (key) {
                if (tmp_heart_1[key] && parseInt(key) < 840) {
                    // for all values before 14:00Uhr 
                    var _d = angular.copy(d);
                    result.push({
                        val: tmp_sleep_1[key],
                        date: _d.minute(parseInt(key))
                    });
                }
            });

            return { data: result };
        }
    }

    return {
        compress: compress,
        getByDate: getByDate
    };
});